// -----------------------------------------------------------------------
// MatchPoint - DKFZ translational registration framework
//
// Copyright (c) German Cancer Research Center (DKFZ),
// Software development for Integrated Diagnostics and Therapy (SIDT).
// ALL RIGHTS RESERVED.
// See mapCopyright.txt or
// http://www.dkfz.de/en/sidt/projects/MatchPoint/copyright.html
//
// This software is distributed WITHOUT ANY WARRANTY; without even
// the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the above copyright notices for more information.
//
//------------------------------------------------------------------------
/*!
// @file
// @version $Revision: 797 $ (last changed revision)
// @date    $Date: 2014-10-10 11:42:05 +0200 (Fr, 10 Okt 2014) $ (last change date)
// @author  $Author: floca $ (last changed by)
// Subversion HeadURL: $HeadURL: http://svn.dkfz-heidelberg.de/sbr/Sources/SBR-Projects/MatchPoint/trunk/Code/Core/include/mapSyncObject.h $
*/


#ifndef __MAP_SYNC_OBJECT_H
#define __MAP_SYNC_OBJECT_H

#include <map>

#include "mapString.h"
#include "mapMetaPropertyBase.h"
#include "mapMetaPropertyAccessor.h"
#include "mapMAPCoreExports.h"

namespace map
{
	namespace deployment
	{

		/** SyncObject that containes all relevant informations
		 * for deployment DLLs to be synced with the host apllication.
		 * This class is used by deployment::Synchronizer and Objects
		 * that implement the DeploymentSyncInterface.*/
		class MAPCore_EXPORT SyncObject
		{
		public:
			typedef core::String SyncPropertyTagType;
			typedef std::map<SyncPropertyTagType, core::MetaPropertyBase::Pointer> SyncPropertyMapType;

			SyncPropertyMapType _map;

			template<typename TPropReturnType> bool getProperty(SyncPropertyTagType tag,
					TPropReturnType& prop) const
			{
				bool result = false;

				SyncPropertyMapType::const_iterator finding = _map.find(tag);

				if (finding != _map.end())
				{
					result = core::unwrapCastedMetaProperty<TPropReturnType>(finding->second, prop);
				}

				return result;
			}

			SyncObject();
			~SyncObject();

		};

	} // end namespace deployment
} // end namespace map

#endif
